@@ -1,9 +1,9 @@
 /*
- * $Id: acl.h,v 1.6.2.11 1996/08/19 19:44:14 wessels Exp $
+ * $Id: acl.h,v 1.32 1997/01/14 19:50:06 wessels Exp $
  *
  * AUTHOR: Duane Wessels
  *
- * SQUID Internet Object Cache  http://www.nlanr.net/Squid/
+ * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
  * --------------------------------------------------------
  *
  *  Squid is the result of efforts by numerous individuals from the
@@ -28,11 +28,13 @@
  */
 
 #define ACL_NAME_SZ 32
+#define BROWSERNAMELEN 128
 
 typedef enum {
     ACL_NONE,
     ACL_SRC_IP,
     ACL_DST_IP,
+    ACL_SRC_DOMAIN,
     ACL_DST_DOMAIN,
     ACL_TIME,
     ACL_URLPATH_REGEX,
@@ -40,7 +42,9 @@ typedef enum {
     ACL_URL_PORT,
     ACL_USER,
     ACL_PROTO,
-    ACL_METHOD
+    ACL_METHOD,
+    ACL_BROWSER,
+    ACL_ENUM_MAX
 } squid_acl;
 
 #define ACL_SUNDAY	0x01
@@ -67,6 +71,17 @@ struct _acl_time_data {
     struct _acl_time_data *next;
 };
 
+struct _acl_name_list {
+    char name[ACL_NAME_SZ];
+    struct _acl_name_list *next;
+};
+
+struct _acl_deny_info_list {
+    char url[MAX_URL];
+    struct _acl_name_list *acl_list;
+    struct _acl_deny_info_list *next;
+};
+
 /* domain data is just a wordlist */
 /* user data is just a wordlist */
 /* port data is just a intlist */
@@ -75,7 +90,7 @@ struct _acl_time_data {
 /* method data is just a intlist */
 
 struct _acl {
-    char name[ACL_NAME_SZ + 1];
+    char name[ACL_NAME_SZ];
     squid_acl type;
     void *data;
     char *cfgline;
@@ -95,13 +110,44 @@ struct _acl_access {
     struct _acl_access *next;
 };
 
-extern int aclCheck _PARAMS((struct _acl_access *, struct in_addr, request_t *));
-extern int aclMatchAcl _PARAMS((struct _acl * acl, struct in_addr c, request_t *));
+typedef enum {
+    ACL_LOOKUP_NONE,
+    ACL_LOOKUP_NEED,
+    ACL_LOOKUP_PENDING,
+    ACL_LOOKUP_DONE
+} acl_lookup_state;
+
+struct _aclCheck_t {
+    struct in_addr src_addr;
+    struct in_addr dst_addr;
+    char src_fqdn[SQUIDHOSTNAMELEN];
+    request_t *request;
+    char ident[ICP_IDENT_SZ];
+    char browser[BROWSERNAMELEN];
+    acl_lookup_state state[ACL_ENUM_MAX];
+};
+
+extern int aclCheck _PARAMS((const struct _acl_access *, aclCheck_t *));
+extern int aclMatchAcl _PARAMS((struct _acl *, aclCheck_t *));
 extern void aclDestroyAccessList _PARAMS((struct _acl_access ** list));
 extern void aclDestroyAcls _PARAMS((void));
 extern void aclParseAccessLine _PARAMS((struct _acl_access **));
 extern void aclParseAclLine _PARAMS((void));
-extern struct _acl *aclFindByName _PARAMS((char *name));
+extern struct _acl *aclFindByName _PARAMS((const char *name));
+extern char *aclGetDenyInfoUrl _PARAMS((struct _acl_deny_info_list **, const char *name));
+extern void aclParseDenyInfoLine _PARAMS((struct _acl_deny_info_list **));
+extern void aclDestroyDenyInfoList _PARAMS((struct _acl_deny_info_list **));
+extern void aclDestroyRegexList _PARAMS((struct _relist * data));
+extern int aclMatchRegex _PARAMS((relist * data, const char *word));
+
+extern void aclParseRegexList _PARAMS((void *curlist, int icase));
 
 extern struct _acl_access *HTTPAccessList;
+extern struct _acl_access *MISSAccessList;
 extern struct _acl_access *ICPAccessList;
+extern struct _acl_deny_info_list *DenyInfoList;
+extern const char *AclMatchedName;
+
+#if DELAY_HACK
+extern struct _acl_access *DelayAccessList;
+#endif
