@@ -1,6 +1,6 @@
 
 /*
- * $Id: client_side.c,v 1.81 1996/12/20 23:22:24 wessels Exp $
+ * $Id: client_side.c,v 1.83 1997/01/18 06:03:26 wessels Exp $
  *
  * DEBUG: section 33    Client-side Routines
  * AUTHOR: Duane Wessels
@@ -440,7 +440,7 @@ icpProcessExpired(int fd, void *data)
 
     entry->refcount++;		/* EXPIRED CASE */
     icpState->entry = entry;
-    icpState->offset = 0;
+    icpState->out_offset = 0;
     /* Register with storage manager to receive updates when data comes in. */
     storeRegister(entry, fd, icpHandleIMSReply, (void *) icpState);
     protoDispatch(fd, url, icpState->entry, icpState->request);
@@ -502,9 +502,6 @@ icpHandleIMSReply(int fd, StoreEntry * entry, void *data)
 	    icpHandleIMSReply,
 	    (void *) icpState);
 	return;
-#if OLD
-    } else if (mem->reply->code == 304 && !BIT_TEST(icpState->request->flags, REQ_IMS)) {
-#endif
     } else if (clientGetsOldEntry(entry, icpState->old_entry, icpState->request)) {
 	/* We initiated the IMS request, the client is not expecting
 	 * 304, so put the good one back.  First, make sure the old entry
