@@ -1,6 +1,6 @@
 
 /*
- * $Id: pass.c,v 1.21 1996/12/13 22:21:33 wessels Exp $
+ * $Id: pass.c,v 1.22 1997/01/21 16:42:05 wessels Exp $
  *
  * DEBUG: section 39    HTTP Passthrough
  * AUTHOR: Duane Wessels
@@ -424,6 +424,10 @@ static void
 passErrorComplete(int fd, char *buf, int size, int errflag, void *passState)
 {
     safe_free(buf);
+    if (passState == NULL) {
+	debug_trap("passErrorComplete: NULL passState\n");
+	return;
+    }
     passClose(passState);
 }
 
@@ -533,12 +537,12 @@ passStart(int fd,
 	    fd_table[fd].ipaddr,
 	    500,
 	    xstrerror());
-	comm_write(passState->client.fd,
+	comm_write(fd,
 	    xstrdup(msg),
 	    strlen(msg),
 	    30,
-	    passErrorComplete,
-	    (void *) passState,
+	    NULL,
+	    NULL,
 	    xfree);
 	return COMM_ERROR;
     }
