@@ -1,10 +1,11 @@
+
 /*
- * $Id: url.c,v 1.19.2.22 1996/08/29 16:37:05 wessels Exp $
+ * $Id: url.c,v 1.51 1996/12/17 07:16:59 wessels Exp $
  *
  * DEBUG: section 23    URL Parsing
  * AUTHOR: Duane Wessels
  *
- * SQUID Internet Object Cache  http://www.nlanr.net/Squid/
+ * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
  * --------------------------------------------------------
  *
  *  Squid is the result of efforts by numerous individuals from the
@@ -30,17 +31,18 @@
 
 #include "squid.h"
 
-char *RequestMethodStr[] =
+const char *RequestMethodStr[] =
 {
     "NONE",
     "GET",
     "POST",
     "PUT",
     "HEAD",
-    "CONNECT"
+    "CONNECT",
+    "TRACE"
 };
 
-char *ProtocolStr[] =
+static char *ProtocolStr[] =
 {
     "NONE",
     "http",
@@ -52,14 +54,14 @@ char *ProtocolStr[] =
 };
 
 static int url_acceptable[256];
-static char hex[17] = "0123456789abcdef";
+static const char *const hex = "0123456789abcdef";
+static int urlDefaultPort _PARAMS((protocol_t p));
 
 /* convert %xx in url string to a character 
  * Allocate a new string and return a pointer to converted string */
 
-char *url_convert_hex(org_url, allocate)
-     char *org_url;
-     int allocate;
+char *
+url_convert_hex(char *org_url, int allocate)
 {
     static char code[] = "00";
     char *url = NULL;
@@ -89,11 +91,13 @@ char *url_convert_hex(org_url, allocate)
 
 /* INIT Acceptable table. 
  * Borrow from libwww2 with Mosaic2.4 Distribution   */
-void urlInitialize()
+void
+urlInitialize(void)
 {
     unsigned int i;
     char *good =
     "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./-_$";
+    debug(23, 5, "urlInitialize: Initializing...\n");
     for (i = 0; i < 256; i++)
 	url_acceptable[i] = 0;
     for (; *good; good++)
@@ -103,10 +107,11 @@ void urlInitialize()
 
 /* Encode prohibited char in string */
 /* return the pointer to new (allocated) string */
-char *url_escape(url)
-     char *url;
+char *
+url_escape(const char *url)
 {
-    char *p, *q;
+    const char *p;
+    char *q;
     char *tmpline = xcalloc(1, MAX_URL);
 
     q = tmpline;
@@ -123,8 +128,8 @@ char *url_escape(url)
     return tmpline;
 }
 
-method_t urlParseMethod(s)
-     char *s;
+method_t
+urlParseMethod(const char *s)
 {
     if (strcasecmp(s, "GET") == 0) {
 	return METHOD_GET;
@@ -136,13 +141,15 @@ method_t urlParseMethod(s)
 	return METHOD_HEAD;
     } else if (strcasecmp(s, "CONNECT") == 0) {
 	return METHOD_CONNECT;
+    } else if (strcasecmp(s, "TRACE") == 0) {
+	return METHOD_TRACE;
     }
     return METHOD_NONE;
 }
 
 
-protocol_t urlParseProtocol(s)
-     char *s;
+protocol_t
+urlParseProtocol(const char *s)
 {
     if (strncasecmp(s, "http", 4) == 0)
 	return PROTO_HTTP;
@@ -164,8 +171,8 @@ protocol_t urlParseProtocol(s)
 }
 
 
-int urlDefaultPort(p)
-     protocol_t p;
+static int
+urlDefaultPort(protocol_t p)
 {
     switch (p) {
     case PROTO_HTTP:
@@ -183,14 +190,13 @@ int urlDefaultPort(p)
     }
 }
 
-request_t *urlParse(method, url)
-     method_t method;
-     char *url;
+request_t *
+urlParse(method_t method, char *url)
 {
-    static char proto[MAX_URL + 1];
-    static char login[MAX_URL + 1];
-    static char host[MAX_URL + 1];
-    static char urlpath[MAX_URL + 1];
+    LOCAL_ARRAY(char, proto, MAX_URL);
+    LOCAL_ARRAY(char, login, MAX_URL);
+    LOCAL_ARRAY(char, host, MAX_URL);
+    LOCAL_ARRAY(char, urlpath, MAX_URL);
     request_t *request = NULL;
     char *t = NULL;
     int port;
@@ -198,7 +204,7 @@ request_t *urlParse(method, url)
     int l;
     proto[0] = host[0] = urlpath[0] = login[0] = '\0';
 
-    if ((l = strlen(url)) > MAX_URL) {
+    if ((l = strlen(url)) > (MAX_URL - 1)) {
 	/* terminate so it doesn't overflow other buffers */
 	*(url + (MAX_URL >> 1)) = '\0';
 	debug(23, 0, "urlParse: URL too large (%d bytes)\n", l);
@@ -228,26 +234,36 @@ request_t *urlParse(method, url)
     }
     for (t = host; *t; t++)
 	*t = tolower(*t);
+    /* remove trailing dots from hostnames */
+    while ((l = strlen(host)) && host[--l] == '.')
+	host[l] = '\0';
     if (port == 0) {
 	debug(23, 0, "urlParse: Invalid port == 0\n");
 	return NULL;
     }
+    /* remove trailing slashes from FTP URLs */
+    if (protocol == PROTO_FTP) {
+	t = urlpath + strlen(urlpath);
+	while (t > urlpath && *(--t) == '/')
+	    *t = '\0';
+    }
     request = get_free_request_t();
     request->method = method;
     request->protocol = protocol;
-    strncpy(request->host, host, SQUIDHOSTNAMELEN);
-    strncpy(request->login, login, MAX_LOGIN_SZ);
-    request->port = port;
-    strncpy(request->urlpath, urlpath, MAX_URL);
+    xstrncpy(request->host, host, SQUIDHOSTNAMELEN);
+    xstrncpy(request->login, login, MAX_LOGIN_SZ);
+    request->port = (u_short) port;
+    xstrncpy(request->urlpath, urlpath, MAX_URL);
+    request->max_age = -1;
+    request->max_forwards = -1;
     return request;
 }
 
-char *urlCanonical(request, buf)
-     request_t *request;
-     char *buf;
+char *
+urlCanonical(const request_t * request, char *buf)
 {
-    static char urlbuf[MAX_URL + 1];
-    static char portbuf[32];
+    LOCAL_ARRAY(char, urlbuf, MAX_URL);
+    LOCAL_ARRAY(char, portbuf, 32);
     if (buf == NULL)
 	buf = urlbuf;
     switch (request->method) {
@@ -261,7 +277,7 @@ char *urlCanonical(request, buf)
 	sprintf(buf, "%s://%s%s%s%s%s",
 	    ProtocolStr[request->protocol],
 	    request->login,
-	    *request->login ? "@" : "",
+	    *request->login ? "@" : null_string,
 	    request->host,
 	    portbuf,
 	    request->urlpath);
@@ -270,26 +286,27 @@ char *urlCanonical(request, buf)
     return buf;
 }
 
-request_t *requestLink(request)
-     request_t *request;
+request_t *
+requestLink(request_t * request)
 {
     request->link_count++;
     return request;
 }
 
-void requestUnlink(request)
-     request_t *request;
+void
+requestUnlink(request_t * request)
 {
     if (request == NULL)
 	return;
     request->link_count--;
-    if (request->link_count == 0)
-	put_free_request_t(request);
+    if (request->link_count)
+	return;
+    safe_free(request->hierarchy.host);
+    put_free_request_t(request);
 }
 
-int matchDomainName(domain, host)
-     char *domain;
-     char *host;
+int
+matchDomainName(const char *domain, const char *host)
 {
     int offset;
     if ((offset = strlen(host) - strlen(domain)) < 0)
@@ -305,10 +322,14 @@ int matchDomainName(domain, host)
     return 0;
 }
 
-int urlCheckRequest(r)
-     request_t *r;
+int
+urlCheckRequest(const request_t * r)
 {
     int rc = 0;
+    if (r->method == METHOD_CONNECT)
+	return 1;
+    if (r->method == METHOD_TRACE)
+	return 1;
     switch (r->protocol) {
     case PROTO_HTTP:
     case PROTO_CACHEOBJ:
